"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = void 0;
const api_1 = require("@deep-foundation/hasura/api");
const sql_1 = require("@deep-foundation/hasura/sql");
const api = new api_1.HasuraApi({
    path: process.env.MIGRATIONS_HASURA_PATH,
    ssl: !!+process.env.MIGRATIONS_HASURA_SSL,
    secret: process.env.MIGRATIONS_HASURA_SECRET,
});
const DEFAULT_SCHEMA = process.env.MIGRATIONS_SCHEMA || 'public';
const DEFAULT_MP_TABLE = process.env.MIGRATIONS_MP_TABLE || 'mp_example__links__mp';
const DEFAULT_GRAPH_TABLE = process.env.MIGRATIONS_GRAPH_TABLE || 'mp_example__links';
const DEFAULT_ID_TYPE_SQL = process.env.MIGRATIONS_ID_TYPE_SQL || 'integer';
exports.up = ({ SCHEMA = DEFAULT_SCHEMA, MP_TABLE = DEFAULT_MP_TABLE, ID_TYPE = DEFAULT_ID_TYPE_SQL, customColumns = '' } = {}) => __awaiter(void 0, void 0, void 0, function* () {
    yield api.sql(sql_1.sql `
    CREATE TABLE ${SCHEMA}."${MP_TABLE}" (id ${ID_TYPE} PRIMARY KEY,item_id ${ID_TYPE},path_item_id ${ID_TYPE},path_item_depth ${ID_TYPE},root_id ${ID_TYPE},position_id text DEFAULT ${SCHEMA}.gen_random_uuid(),group_id ${ID_TYPE},insert_category TEXT${customColumns});
    CREATE SEQUENCE ${SCHEMA}.${MP_TABLE}_id_seq
    AS ${ID_TYPE} START WITH 1 INCREMENT BY 1 NO MINVALUE NO MAXVALUE CACHE 1;
    ALTER SEQUENCE ${SCHEMA}.${MP_TABLE}_id_seq OWNED BY ${SCHEMA}.${MP_TABLE}.id;
    ALTER TABLE ONLY ${SCHEMA}.${MP_TABLE} ALTER COLUMN id SET DEFAULT nextval('${SCHEMA}.${MP_TABLE}_id_seq'::regclass);
  `);
    yield api.query({
        type: 'track_table',
        args: {
            schema: SCHEMA,
            name: MP_TABLE,
        },
    });
});
exports.down = ({ SCHEMA = DEFAULT_SCHEMA, MP_TABLE = DEFAULT_MP_TABLE } = {}) => __awaiter(void 0, void 0, void 0, function* () {
    yield api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: SCHEMA,
                name: MP_TABLE,
            },
            cascade: true,
        },
    });
    yield api.sql(sql_1.sql `
    DROP TABLE ${SCHEMA}."${MP_TABLE}" CASCADE;
  `);
});
//# sourceMappingURL=table.js.map